{ **************************************************************************** }
{ *                                                                          * }
{ * MONKEY JUMP (VTech, Arcade Time & Fun) Simulator                         * }
{ * by Luca "MADrigal" Antignano, January 2003 (revision: August 2021)       * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit gfxinit;
{ this unit contains the graphics initialization (skin-based system)           }

interface

uses
  Forms, Controls, StdCtrls, SysUtils, Classes, Graphics, ExtCtrls, Dialogs,
  Jpeg, Pngimage, IniFiles;

{ * PROCEDURES: SKIN INITIALIZATION                                          * }
procedure Gfx_Game_Init;
procedure Gfx_Manual_Init;

const
  INIFILE = 'Config.ini';

{}IMAGES = 145;

{}DEFAULTHEIGHT    = 664;
{}DEFAULTWIDTH     = 440;
  DEFAULTSKINDIR   = 'Graphics\Skin\';
  DEFAULTWINMODE   = 'Shaped';

  DEFAULTTRCOLOR   = '$00ff00ff';

  DEFAULTBDDIR     = 'Graphics\Backdrop\';
  DEFAULTBDFILE    = 'nil';
  DEFAULTBDSTRETCH = 0;
{}DEFAULTBGCOLOR   = '$00000000';
  DEFAULTOFFSETX   = 'Center';
  DEFAULTOFFSETY   = 'Center';
  DEFAULTLOGOFILE  = 'nil';
  DEFAULTLOGOPOS   = 1;

  DEFAULTMANUALDIR = 'Graphics\Manual\';
{}DEFAULTMANHEIGHT = 415;
{}DEFAULTMANWIDTH  = 275;

var
  IF_File: TIniFile;
  aImages: array [1..IMAGES] of TImage;
{}//sVideoMode: string;

implementation

uses
  manual, splash, unit1;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: SKIN INITIALIZATION                                          * }
{ *                                                                          * }
{ **************************************************************************** }

procedure Gfx_Game_Init;
{ initializes the graphics on the main game form                               }
  var
    // service variables
    i,
    iOffsetX,
    iOffsetY: integer;

    // temporary variables (used to read/write from 'ini file')
    sSkinDir,
    sBackdropDir,
    sBackdropFile,
    sGameOffsetX,
    sGameOffsetY,
{}  sLogoFile: string;
{}//sWindowMode

    // image variables
    sFilename: string;
begin
  // initialises array of images
  with Form1 do
    begin
      aImages[1] := im_background; aImages[2] := im_background_screen_top;

      aImages[3] := btn_game_a_top; aImages[4] := btn_game_a_down;
      aImages[5] := btn_game_b_top; aImages[6] := btn_game_b_down;
      aImages[7] := btn_time_top;   aImages[8] := btn_time_down;
      aImages[9] := btn_set_top;

      aImages[10] := btn_joystick_top;

      aImages[11] := btn_jump_left_down; aImages[12] := btn_jump_left_top;
      aImages[13] := btn_jump_right_down; aImages[14] := btn_jump_right_top;

      aImages[15] := btn_ctrlr_overlay_up;
      aImages[16] := btn_ctrlr_overlay_left;
      aImages[17] := btn_ctrlr_overlay_right;
      aImages[18] := btn_ctrlr_overlay_down;

      aImages[19] := im_panel_bonus_300; aImages[20] := im_panel_bonus_400; aImages[21] := im_panel_bonus_500;
      aImages[22] := im_panel_digit_pos_1; aImages[23] := im_panel_digit_pos_2; aImages[24] := im_panel_digit_pos_3; aImages[25] := im_panel_digit_pos_4; aImages[26] := im_panel_digit_pos_5;
      aImages[27] := im_panel_lives_1; aImages[28] := im_panel_lives_2; aImages[29] := im_panel_lives_3;

      aImages[30] := im_panel_number_0; aImages[31] := im_panel_number_1; aImages[32] := im_panel_number_2; aImages[33] := im_panel_number_3; aImages[34] := im_panel_number_4;
      aImages[35] := im_panel_number_5; aImages[36] := im_panel_number_6; aImages[37] := im_panel_number_7; aImages[38] := im_panel_number_8; aImages[39] := im_panel_number_9;

      aImages[40] := im_hero_01_05; aImages[41] := im_hero_05_05; aImages[42] := im_hero_06_05; aImages[43] := im_hero_07_05; aImages[44] := im_hero_08_05; aImages[45] := im_hero_09_05; aImages[46] := im_hero_10_05; aImages[47] := im_hero_11_05;
      aImages[48] := im_hero_01_04; aImages[49] := im_hero_02_04; aImages[50] := im_hero_03_04; aImages[51] := im_hero_04_04; aImages[52] := im_hero_05_04; aImages[53] := im_hero_06_04; aImages[54] := im_hero_07_04; aImages[55] := im_hero_08_04; aImages[56] := im_hero_09_04; aImages[57] := im_hero_10_04; aImages[58] := im_hero_11_04;
      aImages[59] := im_hero_01_03; aImages[60] := im_hero_02_03; aImages[61] := im_hero_03_03; aImages[62] := im_hero_04_03; aImages[63] := im_hero_05_03; aImages[64] := im_hero_06_03; aImages[65] := im_hero_07_03; aImages[66] := im_hero_08_03; aImages[67] := im_hero_09_03; aImages[68] := im_hero_10_03; aImages[69] := im_hero_11_03;
      aImages[70] := im_hero_01_02; aImages[71] := im_hero_02_02; aImages[72] := im_hero_03_02; aImages[73] := im_hero_04_02; aImages[74] := im_hero_05_02; aImages[75] := im_hero_06_02; aImages[76] := im_hero_07_02; aImages[77] := im_hero_08_02; aImages[78] := im_hero_09_02; aImages[79] := im_hero_10_02; aImages[80] := im_hero_11_02;
      aImages[81] := im_hero_01_01; aImages[82] := im_hero_02_01; aImages[83] := im_hero_03_01; aImages[84] := im_hero_04_01; aImages[85] := im_hero_05_01; aImages[86] := im_hero_06_01; aImages[87] := im_hero_07_01; aImages[88] := im_hero_08_01; aImages[89] := im_hero_09_01; aImages[90] := im_hero_10_01; aImages[91] := im_hero_11_01;

      aImages[92] := im_hero_miss;

      aImages[93] := im_tire_02_05; aImages[94] := im_tire_04_05; aImages[95] := im_tire_06_05; aImages[96] := im_tire_08_05; aImages[97] := im_tire_10_05; aImages[98] := im_tire_12_05;
      aImages[99] := im_tire_00_04; aImages[100] := im_tire_02_04; aImages[101] := im_tire_04_04; aImages[102] := im_tire_06_04; aImages[103] := im_tire_08_04; aImages[104] := im_tire_10_04; aImages[105] := im_tire_12_04;
      aImages[106] := im_tire_00_03; aImages[107] := im_tire_02_03; aImages[108] := im_tire_04_03; aImages[109] := im_tire_06_03; aImages[110] := im_tire_08_03; aImages[111] := im_tire_10_03; aImages[112] := im_tire_12_03;
      aImages[113] := im_tire_00_02; aImages[114] := im_tire_02_02; aImages[115] := im_tire_04_02; aImages[116] := im_tire_06_02; aImages[117] := im_tire_08_02; aImages[118] := im_tire_10_02; aImages[119] := im_tire_12_02;
      aImages[120] := im_tire_00_01; aImages[121] := im_tire_02_01; aImages[122] := im_tire_04_01; aImages[123] := im_tire_06_01; aImages[124] := im_tire_08_01; aImages[125] := im_tire_10_01; aImages[126] := im_tire_12_01;

      aImages[127] := im_kong_1; aImages[128] := im_kong_2; aImages[129] := im_kong_3; aImages[130] := im_kong_4; aImages[131] := im_kong_5; aImages[132] := im_kong_6; aImages[133] := im_kong_7;

      aImages[134] := im_woman_1; aImages[135] := im_woman_2; aImages[136] := im_woman_3; aImages[137] := im_woman_4; aImages[138] := im_woman_5;

      aImages[139] := im_bonus_item_1; aImages[140] := im_bonus_item_2; aImages[141] := im_bonus_item_3; aImages[142] := im_bonus_item_4;

      aImages[143] := im_panel_time_pm; aImages[144] := im_panel_time_colon; aImages[145] := im_panel_bell;
    end;

  // TO BE USED TO GENERATE THE SKIN FILE ONLY (TO BE COMMENTED THEREAFTER)
{  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'Export.ini');

  for i := 1 to IMAGES do
    begin
      if aImages[i].Picture.Graphic = nil then
        sFilename := 'nil'
      else
        if aImages[i].Picture.Graphic is TJPEGImage then
          sFilename := aImages[i].Name + '.jpg'
        else
          if aImages[i].Picture.Graphic is TBitmap then
            sFilename := aImages[i].Name + '.bmp'
          else
            if aImages[i].Picture.Graphic is TPNGObject then
              sFilename := aImages[i].Name + '.png';

      IF_file.WriteString('Image' + IntToStr(i),  'Filename', sFilename);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Height',   aImages[i].Height);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Left',     aImages[i].Left);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Top',      aImages[i].Top);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Width',    aImages[i].Width);
    end;

  IF_File.Destroy; }

  // loads configuration settings from the Config.ini file
  if not FileExists(ExtractFilePath(ParamStr(0)) + INIFILE) then
    begin
      MessageDlg('Configuration file (Config.ini) not found. Please reinstall the game.', mtError, [mbOK], 0);
      Application.Terminate;
    end
  else
    begin
      // reads init values from 'ini file' -- with defaults if no 'ini file' found
      IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

      // game form initialization (game graphics and backdrop)
      with Form1 do
        begin
          sSkinDir     := IncludeTrailingPathDelimiter(IF_file.ReadString('Common', 'SkinDir', DEFAULTSKINDIR));
          ClientHeight := IF_file.ReadInteger('Common', 'GameHeight', DEFAULTHEIGHT);
          ClientWidth  := IF_file.ReadInteger('Common', 'GameWidth',  DEFAULTWIDTH);

          // use Fullscreen or Shaped mode
{}        if bFullscreenMode then
            begin                                                               // Fullscreen
              // sets backdrop image and background colour
              sBackdropDir  := IncludeTrailingPathDelimiter(IF_file.ReadString('Fullscreen', 'BackdropDir',  DEFAULTBDDIR));
              sBackdropFile := IF_file.ReadString('Fullscreen', 'BackdropFile', DEFAULTBDFILE);

              if (sBackdropFile <> '')
                and (sBackdropFile <> 'nil') then
                if FileExists(sBackdropDir + sBackdropFile) then
                  begin
                    im_backdrop.Picture.LoadFromFile(sBackdropDir + sBackdropFile);
                    im_backdrop.Stretch := Boolean(IF_file.ReadInteger('Fullscreen', 'BackdropStretch', DEFAULTBDSTRETCH));
{}                  im_backdrop.Visible := true;
                  end;

              Color := StrToInt(IF_file.ReadString('Fullscreen', 'BgColor', DEFAULTBGCOLOR));
              TransparentColor := false;

              // sets new coordinates for the handheld game graphics
              sGameOffsetX := IF_file.ReadString('Fullscreen', 'GameOffsetX', DEFAULTOFFSETX);
              sGameOffsetY := IF_file.ReadString('Fullscreen', 'GameOffsetY', DEFAULTOFFSETY);

              if StrToIntDef(sGameOffsetX, 0) <> 0 then iOffsetX := StrToInt(sGameOffsetX) else iOffsetX := Trunc((Screen.Width  - ClientWidth)  / 2);
              if StrToIntDef(sGameOffsetY, 0) <> 0 then iOffsetY := StrToInt(sGameOffsetY) else iOffsetY := Trunc((Screen.Height - ClientHeight) / 2);

              // sets the form as full screen
              ClientWidth  := Screen.Width;
              ClientHeight := Screen.Height;

              // sets logo (watermark) image and its position
              sLogoFile := IF_file.ReadString('Fullscreen', 'LogoFile', DEFAULTLOGOFILE);

              if (sLogoFile <> '')
                and (sLogoFile <> 'nil') then
                if FileExists(sBackdropDir + sLogoFile) then
                  with Form1 do
                    begin
                      im_backdrop_logo.Picture.LoadFromFile(sBackdropDir + sLogoFile);
                      im_backdrop_logo.AutoSize := true;
{}                    im_backdrop_logo.Visible := true;

                      case IF_file.ReadInteger('Fullscreen', 'LogoPosition', DEFAULTLOGOPOS) of
                        0: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := 0; end; // top-left
                        1: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := 0; end; // top-right
                        2: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-left
                        3: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-right
                      end; // case..of
                    end;
            end
          else                                                                  // Shaped
            begin
              // sets coordinates for the handheld game graphics
              iOffsetX := 0;
              iOffsetY := 0;

              // sets the form as 'shaped'
              Color                 := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColorValue := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColor      := true;

              // disables the backdrop
              im_backdrop.Visible := false;
              im_backdrop_logo.Visible := false;
            end;
        end;

      // game skin initialization
      for i := 1 to IMAGES do
        begin
          sFilename := IF_file.ReadString('Image' + IntToStr(i), 'Filename', '');

          if (sFilename <> '')
            and (sFilename <> 'nil') then
            if FileExists(sSkinDir + sFilename) then
              // loads picture into the image
              aImages[i].Picture.LoadFromFile(sSkinDir + sFilename)
            else
              // image expected but not found -> creates a coloured square as a placeholder
              begin
                aImages[i].Picture.Bitmap.Create;
                aImages[i].Picture.Bitmap.Width  := 10;
                aImages[i].Picture.Bitmap.Height := 10;
                aImages[i].Picture.Bitmap.Canvas.Create;
                aImages[i].Picture.Bitmap.Canvas.Pen.Color := 0;
                aImages[i].Picture.Bitmap.Canvas.Pen.Style := psSolid;
                aImages[i].Picture.Bitmap.Canvas.Brush.Color := $000000ff;
                aImages[i].Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
              end;

          aImages[i].Height := IF_file.ReadInteger('Image' + IntToStr(i), 'Height', 0);
          aImages[i].Left   := IF_file.ReadInteger('Image' + IntToStr(i), 'Left',   0) + iOffsetX;
          aImages[i].Top    := IF_file.ReadInteger('Image' + IntToStr(i), 'Top',    0) + iOffsetY;
          aImages[i].Width  := IF_file.ReadInteger('Image' + IntToStr(i), 'Width',  0);
        end;

      // close Inifile
      IF_File.Destroy;
    end;

  // initializes the on-screen game menu
  with Form1 do
    begin
{}    btn_game_menu_top.Visible := bFullscreenMode;
{}    btn_game_menu_background.Visible := bFullscreenMode;

      btn_game_menu_background.Height := 32;
      btn_game_menu_background.Width := 36;
    end;
end;





procedure Gfx_Manual_Init;
{ initializes the graphics of the instructions manual form                     }
  var
    // service variables
    i: integer;

    // temporary variables (used to read/write from 'ini file')
    sManualDir: string;

    // image variables
    sFilename: string;
begin
  // reads init values from 'ini file' -- with defaults if no 'ini file' found
  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

  // manual form initialization (graphics and sizes)
  with ManualForm do
    begin
      sManualDir   := IncludeTrailingPathDelimiter(IF_file.ReadString('Manual', 'ManualDir', DEFAULTMANUALDIR));

      ClientHeight := IF_file.ReadInteger('Manual', 'ManualHeight', DEFAULTMANHEIGHT);
      ClientWidth  := IF_file.ReadInteger('Manual', 'ManualWidth',  DEFAULTMANWIDTH);

      im_page_top.Left   := 0;
      im_page_top.Top    := 0;
      im_page_top.Height := ClientHeight;
      im_page_top.Width  := ClientWidth;

      sh_page_top.Left   := 0;
      sh_page_top.Top    := 0;
      sh_page_top.Height := ClientHeight;
      sh_page_top.Width  := ClientWidth;
    end;

  // instructions manual skin initialization
  for i := 1 to MANUALPAGES do
    begin
      sFilename := IntToStr(i) + '.bmp';

      if FileExists(sManualDir + sFilename) then
          // loads picture into the image
          aIm_pages[i].Picture.LoadFromFile(sManualDir + sFilename)
        else
          // image expected but not found -> creates a coloured square as a placeholder
          begin
            aIm_pages[i].Picture.Bitmap.Create;
            aIm_pages[i].Picture.Bitmap.Width  := 10;
            aIm_pages[i].Picture.Bitmap.Height := 10;
            aIm_pages[i].Picture.Bitmap.Canvas.Create;
            aIm_pages[i].Picture.Bitmap.Canvas.Pen.Color := 0;
            aIm_pages[i].Picture.Bitmap.Canvas.Pen.Style := psSolid;
            aIm_pages[i].Picture.Bitmap.Canvas.Brush.Color := $000000ff;
            aIm_pages[i].Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
          end;

      with ManualForm do
        begin
          aIm_pages[i].Height := ClientHeight;
          aIm_pages[i].Width  := ClientWidth;
        end;
    end;

  // close Inifile
  IF_File.Destroy;
end;





end.
