{ **************************************************************************** }
{ *                                                                          * }
{ * MONKEY JUMP (VTech, Arcade Time & Fun) Simulator                         * }
{ * by Luca "MADrigal" Antignano, January 2003 (revision: August 2021)       * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;    im_background_screen_top: TImage;

    btn_game_a_down:          TImage;    btn_game_a_top:           TImage;
    btn_game_b_down:          TImage;    btn_game_b_top:           TImage;
    btn_time_down:            TImage;    btn_time_top:             TImage;
                                         btn_set_top:              TImage;

    btn_joystick_top:         TImage;

    btn_jump_right_down:      TImage;    btn_jump_right_top:       TImage;
    btn_jump_left_down:       TImage;    btn_jump_left_top:        TImage;

    btn_ctrlr_overlay_up:     TImage;
    btn_ctrlr_overlay_left:   TImage;
    btn_ctrlr_overlay_right:  TImage;
    btn_ctrlr_overlay_down:   TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage; im_panel_digit_pos_5: TImage;

    im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_bell: TImage;

    im_panel_bonus_300: TImage; im_panel_bonus_400: TImage; im_panel_bonus_500: TImage;

    im_panel_lives_1: TImage; im_panel_lives_2: TImage; im_panel_lives_3: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // on-screen items
    im_hero_01_05: TImage;                                                                      im_hero_05_05: TImage; im_hero_06_05: TImage; im_hero_07_05: TImage; im_hero_08_05: TImage; im_hero_09_05: TImage; im_hero_10_05: TImage; im_hero_11_05: TImage;
    im_hero_01_04: TImage; im_hero_02_04: TImage; im_hero_03_04: TImage; im_hero_04_04: TImage; im_hero_05_04: TImage; im_hero_06_04: TImage; im_hero_07_04: TImage; im_hero_08_04: TImage; im_hero_09_04: TImage; im_hero_10_04: TImage; im_hero_11_04: TImage;
    im_hero_01_03: TImage; im_hero_02_03: TImage; im_hero_03_03: TImage; im_hero_04_03: TImage; im_hero_05_03: TImage; im_hero_06_03: TImage; im_hero_07_03: TImage; im_hero_08_03: TImage; im_hero_09_03: TImage; im_hero_10_03: TImage; im_hero_11_03: TImage;
    im_hero_01_02: TImage; im_hero_02_02: TImage; im_hero_03_02: TImage; im_hero_04_02: TImage; im_hero_05_02: TImage; im_hero_06_02: TImage; im_hero_07_02: TImage; im_hero_08_02: TImage; im_hero_09_02: TImage; im_hero_10_02: TImage; im_hero_11_02: TImage;
    im_hero_01_01: TImage; im_hero_02_01: TImage; im_hero_03_01: TImage; im_hero_04_01: TImage; im_hero_05_01: TImage; im_hero_06_01: TImage; im_hero_07_01: TImage; im_hero_08_01: TImage; im_hero_09_01: TImage; im_hero_10_01: TImage; im_hero_11_01: TImage;

    im_hero_miss: TImage;

                           im_tire_02_05: TImage;                        im_tire_04_05: TImage;                        im_tire_06_05: TImage;                        im_tire_08_05: TImage;                        im_tire_10_05: TImage;                        im_tire_12_05: TImage;
    im_tire_00_04: TImage; im_tire_02_04: TImage;                        im_tire_04_04: TImage;                        im_tire_06_04: TImage;                        im_tire_08_04: TImage;                        im_tire_10_04: TImage;                        im_tire_12_04: TImage;
    im_tire_00_03: TImage; im_tire_02_03: TImage;                        im_tire_04_03: TImage;                        im_tire_06_03: TImage;                        im_tire_08_03: TImage;                        im_tire_10_03: TImage;                        im_tire_12_03: TImage;
    im_tire_00_02: TImage; im_tire_02_02: TImage;                        im_tire_04_02: TImage;                        im_tire_06_02: TImage;                        im_tire_08_02: TImage;                        im_tire_10_02: TImage;                        im_tire_12_02: TImage;
    im_tire_00_01: TImage; im_tire_02_01: TImage;                        im_tire_04_01: TImage;                        im_tire_06_01: TImage;                        im_tire_08_01: TImage;                        im_tire_10_01: TImage;                        im_tire_12_01: TImage;

    im_kong_1: TImage; im_kong_2: TImage; im_kong_3: TImage; im_kong_4: TImage; im_kong_5: TImage; im_kong_6: TImage; im_kong_7: TImage;

    im_woman_1: TImage; im_woman_2: TImage; im_woman_3: TImage; im_woman_4: TImage; im_woman_5: TImage;

    im_bonus_item_1: TImage; im_bonus_item_2: TImage; im_bonus_item_3: TImage; im_bonus_item_4: TImage;

{ * TIMERS                                                                   * }
    Timer_Alarm:          TTimer;
    Timer_Clock:          TTimer;
    Timer_Game:           TTimer;
    Timer_Game_Over:      TTimer;
    Timer_Game_Start:     TTimer;
    Timer_HiScore:        TTimer;
    Timer_Miss:           TTimer;
    Timer_Stage_Complete: TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_controller:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                 (Sender: TObject);
    procedure FormActivate               (Sender: TObject);
    procedure FormClose                  (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Display_Ctrl_Overlay;

    procedure Vid_Show_All               (b: boolean);

    procedure Vid_Show_All_Bonus_Item    (b: boolean);
    procedure Vid_Show_All_Hero          (b: boolean);
    procedure Vid_Show_All_Kong          (b: boolean);
    procedure Vid_Show_All_Panel_Bonus   (b: boolean);
    procedure Vid_Show_All_Tire          (b: boolean);
    procedure Vid_Show_All_Woman         (b: boolean);

    procedure Vid_Show_Hero              (b: boolean);

    procedure Vid_Show_Panel_Bonus       (var p: integer);
    procedure Vid_Show_Panel_HiScore     (var p: integer; b: boolean);
    procedure Vid_Show_Panel_Lives       (var p: integer);
    procedure Vid_Show_Panel_Score       (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                   (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove     (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_jump_left_topMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_jump_left_topMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_jump_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_jump_right_topMouseUp  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_ctrlr_overlay_upMouseDown(Sender:TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseDown(Sender:TObject;Button:TMouseButton;Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseDown(Sender:TObject;Button:TMouseButton;Shift:TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseDown(Sender:TObject;Button:TMouseButton;Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                  (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;           procedure Inp_game_a_up;
    procedure Inp_game_b_down;           procedure Inp_game_b_up;
    procedure Inp_time_down;             procedure Inp_time_up;

    procedure Inp_up_down;
    procedure Inp_left_down;
    procedure Inp_right_down;
    procedure Inp_down_down;
    procedure Inp_jump_down;

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Set_Mode;
    procedure Gam_Set_Stage;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_AlarmTimer           (Sender: TObject);
    procedure Timer_ClockTimer           (Sender: TObject);
    procedure Timer_GameTimer            (Sender: TObject);
    procedure Timer_Game_OverTimer       (Sender: TObject);
    procedure Timer_Game_StartTimer      (Sender: TObject);
    procedure Timer_HiScoreTimer         (Sender: TObject);
    procedure Timer_MissTimer            (Sender: TObject);
    procedure Timer_Stage_CompleteTimer  (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup             (Sender: TObject);
    procedure pm_1_aboutClick            (Sender: TObject);
    procedure pm_2_readmeClick           (Sender: TObject);
    procedure pm_3_whatsnewClick         (Sender: TObject);
    procedure pm_4_manualClick           (Sender: TObject);
    procedure pm_6_configClick           (Sender: TObject);
    procedure pm_7_keysClick             (Sender: TObject);
    procedure pm_8_hiscoreClick          (Sender: TObject);
  {}procedure pm_10_fullscreenClick      (Sender: TObject);
  {}procedure pm_11_splashscreenClick    (Sender: TObject);
  {}procedure pm_12_controllerClick      (Sender: TObject);
  {}procedure pm_14_exitClick            (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick     (Sender: TObject);

    procedure la_menu_itemMouseEnter     (Sender: TObject);
    procedure la_menu_itemMouseLeave     (Sender: TObject);
    procedure la_menu_itemMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu      (b: boolean);
end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Monkey Jump (VTech, Arcade Time & Fun)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY       = 'Software\MADrigal\MonkeyJ50';                                // registry key

  HINT_CLICK    = 'click or press';
  HINT_GAME_A   = 'to start game A';
  HINT_GAME_B   = 'to start game B';
  HINT_TIME     = 'to switch mode';
  HINT_SET      = 'button disabled';
  HINT_JOYSTICK = 'activate the controller overlays or use';

{}MENU_ITEMS = 14;
  NUM_KEYS = 9;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save settings)

{ * sound management * }
  pfs_bonus1,
  pfs_bonus2,
  pfs_gameover,
  pfs_jump,
  pfs_miss,
  pfs_move,
  pfs_moveup,
  pfs_powerup,
  pfs_start:               PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..5]        of TImage;
  aIm_panel_number:        array[0..9]        of TImage;
  aIm_panel_bonus:         array[3..5]        of TImage;
  aIm_panel_lives:         array[1..3]        of TImage;

  aIm_hero:                array[1..11, 1..5] of TImage;
  aIm_tire:                array[0..12, 1..5] of TImage;
  aIm_kong:                array[1..7]        of TImage;
  aIm_woman:               array[1..5]        of TImage;
  aIm_bonus_item:          array[1..4]        of TImage;

{ * arrays of numbers *}
  aTire:                   array[0..12, 1..5] of integer;                       // how tires move (0=off; 1=slow; 2=fast; 3=jumped)

{ * arrays of mixed types *}
  aKeys:                   array[1..NUM_KEYS] of record                         // keys management (values)
                                                   int:  integer;
                                                   word: word;
                                                end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=ACL, 1=time, etc.
  iScore,                                                                       // 0-99990
  iHiScoreA,                                                                    // 0-99990
  iHiScoreB,                                                                    // 0-99990

  iHeroPosX, iHeroPosY,                                                         // position of hero (x in 1-11, even when jumping; y in 1-5)
  iDir,                                                                         // which direction hero goes (1=up, 2=left, 3=right, 4=down, 5=jump left, 6=jump right)

  iKongPos,                                                                     // position of kong (1-7, 1=low, 5=on top of tree, 7=on topmost bench)

  iLivesLeft,                                                                   // lives left
  iBonus,                                                                       // current value of bonus (500, 400 or 300)

  iStage,                                                                       // current stage
  iMaxTires,                                                                    // maximum tires simultaneously on screen

  iLoop,                                                                        // various "looping" variables...
  iLoopHiScore,
  iLoopGameOver,
  iLoopGameStart,
  iLoopMiss,
  iLoopPowerup,
  iLoopStageComplete
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible
  bDisplayCtrlOverlays,                                                         // true if the controller overlay should be displayed

  bTimePressed,                                                                 // true while 'Time'   button is being kept pressed
  bGameAPressed,                                                                // true while 'Game A' button is being kept pressed
  bGameBPressed,                                                                // true while 'Game B' button is being kept pressed
  bControls                                                                     // true when hero can be operated by player
                         : boolean;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4; aIm_panel_digit_pos[5] := im_panel_digit_pos_5;

  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  aIm_panel_lives[1] := im_panel_lives_1; aIm_panel_lives[2] := im_panel_lives_2; aIm_panel_lives[3] := im_panel_lives_3;

  aIm_panel_bonus[3] := im_panel_bonus_300; aIm_panel_bonus[4] := im_panel_bonus_400; aIm_panel_bonus[5] := im_panel_bonus_500;

  // on-screen items
  aIm_hero[1, 5] := im_hero_01_05;                                                                                                    aIm_hero[5, 5] := im_hero_05_05; aIm_hero[6, 5] := im_hero_06_05; aIm_hero[7, 5] := im_hero_07_05; aIm_hero[8, 5] := im_hero_08_05; aIm_hero[9, 5] := im_hero_09_05; aIm_hero[10, 5] := im_hero_10_05; aIm_hero[11, 5] := im_hero_11_05;
  aIm_hero[1, 4] := im_hero_01_04; aIm_hero[2, 4] := im_hero_02_04; aIm_hero[3, 4] := im_hero_03_04; aIm_hero[4, 4] := im_hero_04_04; aIm_hero[5, 4] := im_hero_05_04; aIm_hero[6, 4] := im_hero_06_04; aIm_hero[7, 4] := im_hero_07_04; aIm_hero[8, 4] := im_hero_08_04; aIm_hero[9, 4] := im_hero_09_04; aIm_hero[10, 4] := im_hero_10_04; aIm_hero[11, 4] := im_hero_11_04;
  aIm_hero[1, 3] := im_hero_01_03; aIm_hero[2, 3] := im_hero_02_03; aIm_hero[3, 3] := im_hero_03_03; aIm_hero[4, 3] := im_hero_04_03; aIm_hero[5, 3] := im_hero_05_03; aIm_hero[6, 3] := im_hero_06_03; aIm_hero[7, 3] := im_hero_07_03; aIm_hero[8, 3] := im_hero_08_03; aIm_hero[9, 3] := im_hero_09_03; aIm_hero[10, 3] := im_hero_10_03; aIm_hero[11, 3] := im_hero_11_03;
  aIm_hero[1, 2] := im_hero_01_02; aIm_hero[2, 2] := im_hero_02_02; aIm_hero[3, 2] := im_hero_03_02; aIm_hero[4, 2] := im_hero_04_02; aIm_hero[5, 2] := im_hero_05_02; aIm_hero[6, 2] := im_hero_06_02; aIm_hero[7, 2] := im_hero_07_02; aIm_hero[8, 2] := im_hero_08_02; aIm_hero[9, 2] := im_hero_09_02; aIm_hero[10, 2] := im_hero_10_02; aIm_hero[11, 2] := im_hero_11_02;
  aIm_hero[1, 1] := im_hero_01_01; aIm_hero[2, 1] := im_hero_02_01; aIm_hero[3, 1] := im_hero_03_01; aIm_hero[4, 1] := im_hero_04_01; aIm_hero[5, 1] := im_hero_05_01; aIm_hero[6, 1] := im_hero_06_01; aIm_hero[7, 1] := im_hero_07_01; aIm_hero[8, 1] := im_hero_08_01; aIm_hero[9, 1] := im_hero_09_01; aIm_hero[10, 1] := im_hero_10_01; aIm_hero[11, 1] := im_hero_11_01;

                                   aIm_tire[2, 5] := im_tire_02_05; aIm_tire[4, 5] := im_tire_04_05; aIm_tire[6, 5] := im_tire_06_05; aIm_tire[8, 5] := im_tire_08_05; aIm_tire[10, 5] := im_tire_10_05; aIm_tire[12, 5] := im_tire_12_05;
  aIm_tire[0, 4] := im_tire_00_04; aIm_tire[2, 4] := im_tire_02_04; aIm_tire[4, 4] := im_tire_04_04; aIm_tire[6, 4] := im_tire_06_04; aIm_tire[8, 4] := im_tire_08_04; aIm_tire[10, 4] := im_tire_10_04; aIm_tire[12, 4] := im_tire_12_04;
  aIm_tire[0, 3] := im_tire_00_03; aIm_tire[2, 3] := im_tire_02_03; aIm_tire[4, 3] := im_tire_04_03; aIm_tire[6, 3] := im_tire_06_03; aIm_tire[8, 3] := im_tire_08_03; aIm_tire[10, 3] := im_tire_10_03; aIm_tire[12, 3] := im_tire_12_03;
  aIm_tire[0, 2] := im_tire_00_02; aIm_tire[2, 2] := im_tire_02_02; aIm_tire[4, 2] := im_tire_04_02; aIm_tire[6, 2] := im_tire_06_02; aIm_tire[8, 2] := im_tire_08_02; aIm_tire[10, 2] := im_tire_10_02; aIm_tire[12, 2] := im_tire_12_02;
  aIm_tire[0, 1] := im_tire_00_01; aIm_tire[2, 1] := im_tire_02_01; aIm_tire[4, 1] := im_tire_04_01; aIm_tire[6, 1] := im_tire_06_01; aIm_tire[8, 1] := im_tire_08_01; aIm_tire[10, 1] := im_tire_10_01; aIm_tire[12, 1] := im_tire_12_01;

  aIm_kong[1] := im_kong_1; aIm_kong[2] := im_kong_2; aIm_kong[3] := im_kong_3; aIm_kong[4] := im_kong_4; aIm_kong[5] := im_kong_5; aIm_kong[6] := im_kong_6; aIm_kong[7] := im_kong_7;

  aIm_woman[1] := im_woman_1; aIm_woman[2] := im_woman_2; aIm_woman[3] := im_woman_3; aIm_woman[4] := im_woman_4; aIm_woman[5] := im_woman_5;

  aIm_bonus_item[1] := im_bonus_item_1; aIm_bonus_item[2] := im_bonus_item_2; aIm_bonus_item[3] := im_bonus_item_3; aIm_bonus_item[4] := im_bonus_item_4;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A"       (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B"       (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"         (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 37);                             // "Up"           (default: "UP")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 39);                             // "Left"         (default: "LEFT")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 40);                             // "Right"        (default: "RIGHT")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 38);                             // "Down"         (default: "DOWN")
  aKeys[8].int := iReg.ReadInteger('', 'Key8', 42);                             // "Jump (left)"  (default: "CTRL")
  aKeys[9].int := iReg.ReadInteger('', 'Key9', 41);                             // "Jump (right)" (default: "CTRL")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to NUM_KEYS do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint      := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint      := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint        := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_set_top.Hint         := HINT_SET;                                                       // "Set"

  btn_joystick_top.Hint    := HINT_JOYSTICK + ' ' + aKeyHints[aKeys[4].int] + ', ' // "Up"
                                                  + aKeyHints[aKeys[5].int] + ', ' // "Left"
                                                  + aKeyHints[aKeys[6].int] + ', ' // "Right"
                                                  + aKeyHints[aKeys[7].int];       // "Down"

  btn_jump_left_top.Hint       := HINT_CLICK + ' ' + aKeyHints[aKeys[8].int];   // "Jump (left)"
  btn_jump_left_top.Hint       := HINT_CLICK + ' ' + aKeyHints[aKeys[9].int];   // "Jump (right)"

  btn_ctrlr_overlay_up.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];   // "Up"
  btn_ctrlr_overlay_left.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];   // "Left"
  btn_ctrlr_overlay_right.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];   // "Right"
  btn_ctrlr_overlay_down.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];   // "Down"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}aMenuItem[12] := pm_12_controller;
{}aMenuItem[13] := pm_13_n3;
{}aMenuItem[14] := pm_14_exit;

  // controller overlay setting
{}pm_12_controller.Checked  := iReg.ReadBool('', 'CtrlrOverlay', true);
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

  // controller overlay
  Vid_Display_Ctrl_Overlay;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bTimepressed  := false;                                                       // 'Time'   button is not currently pressed
  bGameAPressed := false;                                                       // 'Game A' button is not currently pressed
  bGameBPressed := false;                                                       // 'Game B' button is not currently pressed

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_bonus1);   FSOUND_Sample_Free(pfs_bonus2);
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_jump);
  FSOUND_Sample_Free(pfs_miss);     FSOUND_Sample_Free(pfs_move);
  FSOUND_Sample_Free(pfs_moveup);   FSOUND_Sample_Free(pfs_powerup);
  FSOUND_Sample_Free(pfs_start);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Display_Ctrl_Overlay;
{ displays or hides the controller overlay images                              }
begin
  btn_ctrlr_overlay_up.Visible    := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_left.Visible  := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_right.Visible := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_down.Visible  := bDisplayCtrlOverlays;

  btn_joystick_top.Visible := (not bDisplayCtrlOverlays);
end;





procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides nearly all sprites (except score panel)                       }
  var
    i: integer;
begin
  for i := 1 to 5 do
    aIm_panel_digit_pos[i].Visible := true;

  im_panel_bell.Visible := b;
  im_panel_time_pm.Visible := b;
  im_panel_time_colon.Visible := b;

  i := 3 * Ord(b); Vid_Show_Panel_Lives(i);
  i := 88880; Vid_Show_Panel_Score(i, b);

  Vid_Show_All_Panel_Bonus(b);
  Vid_Show_All_Hero(b);
  Vid_Show_All_Tire(b);
  Vid_Show_All_Kong(b);
  Vid_Show_All_Woman(b);
  Vid_Show_All_Bonus_Item(b);
end;





procedure TForm1.Vid_Show_All_Bonus_Item(b: boolean);
{ displays/hides all bonus items sprites                                       }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_bonus_item[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all player character sprites                                  }
  var
    i, j: integer;
begin
  for i := 1 to 11 do
    for j := 1 to 4 do
      aIm_hero[i, j].Visible := b;

  for i := 5 to 11 do
    aIm_hero[i, 5].Visible := b;

  aIm_hero[1, 5].Visible := b;

  im_hero_miss.Visible := b;
end;





procedure TForm1.Vid_Show_All_Kong(b: boolean);
{ displays/hides all kong sprites                                              }
  var
    i: integer;
begin
  for i := 1 to 7 do
    aIm_kong[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Panel_Bonus(b: boolean);
{ displays/hides all bonus panel sprites                                       }
  var
    i: integer;
begin
  for i := 3 to 5 do
    aIm_panel_bonus[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Tire(b: boolean);
{ displays/hides all tires sprites                                             }
  var
    i, j: integer;
begin
  for i := 2 to 12 do
    if not Odd(i) then
      aIm_tire[i, 5].Visible := b;

  for i := 0 to 12 do
    for j := 1 to 4 do
      if not Odd(i) then
        aIm_tire[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Woman(b: boolean);
{ displays/hides all woman sprites                                             }
  var
    i: integer;
begin
  for i := 1 to 5 do
    aIm_woman[i].Visible := b;
end;





procedure TForm1.Vid_Show_Hero(b: boolean);
{ displays/hides player character as stated on his iHeroPosX and iHeroPosY     }
{ coordinates                                                                  }
begin
  aIm_hero[iHeroPosX, iHeroPosY].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Bonus(var p: integer);
{ displays bonus score panel (300, 400 or 500)                                 }
  var
    i: integer;
begin
  for i := 3 to 5 do
    aIm_panel_bonus[i].Visible := (i = p);                                      // displays either of bonus scores
end;





procedure TForm1.Vid_Show_Panel_HiScore(var p: integer; b: boolean);
{ same as Vid_ShowPanelScore but adds zeroes to blank spaces                   }
  var
    i: integer;
begin
  while p >= 100000 do                                                          // prevents displayed number from being above 99990
    Dec(p, 100000);

  if b then                                                                     // display or hide high-score?
    begin                                                                         // display
      if p > 9999 then aIm_panel_digit_pos[1].Picture := aIm_panel_number[p div 10000 mod 10].Picture else aIm_panel_digit_pos[1].Picture := aIm_panel_number[0].Picture; // tens of thousands
      if p >  999 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[p div  1000 mod 10].Picture else aIm_panel_digit_pos[2].Picture := aIm_panel_number[0].Picture; // thousands
      if p >   99 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[p div   100 mod 10].Picture else aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture; // hundreds
      if p >    9 then aIm_panel_digit_pos[4].Picture := aIm_panel_number[p div    10 mod 10].Picture else aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture; // tens
                       aIm_panel_digit_pos[5].Picture := aIm_panel_number[0].Picture;                                                                                     // units
    end
  else                                                                            // hide
    for i := 1 to 5 do
      aIm_panel_digit_pos[i].Picture := nil;

  for i := 1 to 5 do
    aIm_panel_digit_pos[i].Visible := true;
end;





procedure TForm1.Vid_Show_Panel_Lives(var p: integer);
{ displays as many sprites as p on the lives counter (0=nothing visible)       }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_panel_lives[i].Visible := (i <= p);                                     // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score into sprites, and displays/hides them                       }
  var
    i, t: integer;
begin
  t := p;

  while t >= 100000 do                                                          // prevents displayed score from being above 99990
    Dec(t, 100000);

  if b then                                                                     // display or hide score
    begin                                                                         // display
      if t > 9999 then aIm_panel_digit_pos[1].Picture := aIm_panel_number[t div 10000 mod 10].Picture else aIm_panel_digit_pos[1].Picture := nil; // tens of thousands
      if t >  999 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div  1000 mod 10].Picture else aIm_panel_digit_pos[2].Picture := nil; // thousands
      if t >   99 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div   100 mod 10].Picture else aIm_panel_digit_pos[3].Picture := nil; // hundreds
      if t >    9 then aIm_panel_digit_pos[4].Picture := aIm_panel_number[t div    10 mod 10].Picture else aIm_panel_digit_pos[4].Picture := nil; // tens
                       aIm_panel_digit_pos[5].Picture := aIm_panel_number[0].Picture;                                                             // units
    end
  else                                                                            // hide
    for i := 1 to 5 do
      aIm_panel_digit_pos[i].Picture := nil;

  for i := 1 to 5 do
    aIm_panel_digit_pos[i].Visible := true;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_jump_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ jump (left) button: PRESS                                                    }
begin
  if Button = mbLeft then
    begin
      btn_jump_left_down.Visible := true;
      Inp_jump_down;
    end;
end;





procedure TForm1.btn_jump_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ jump (left) button: RELEASE                                                  }
begin
  if Button = mbLeft then
    btn_jump_left_down.Visible := false;
end;





procedure TForm1.btn_jump_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ jump (right) button: PRESS                                                   }
begin
  if Button = mbLeft then
    begin
      btn_jump_right_down.Visible := true;
      Inp_jump_down;
    end;
end;





procedure TForm1.btn_jump_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ jump (right) button: RELEASE                                                 }
begin
  if Button = mbLeft then
    btn_jump_right_down.Visible := false;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ up button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_up_down;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_ctrlr_overlay_downMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ down button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_down_down;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time

  if Key = aKeys[4].word then Inp_up_down     else                              // Up
  if Key = aKeys[5].word then Inp_left_down   else                              // Left
  if Key = aKeys[6].word then Inp_right_down  else                              // Right
  if Key = aKeys[7].word then Inp_down_down   else                              // Down

  if Key = aKeys[8].word then                                                   // Jump (left)
    begin
      btn_jump_left_down.Visible := true;
      Inp_jump_down;
    end                                       else
  if Key = aKeys[9].word then                                                   // Jump (right)
    begin
      btn_jump_right_down.Visible := true;
      Inp_jump_down;
    end;
end;





procedure TForm1.FormKeyUp(Sender:TObject; var Key:Word; Shift:TShiftState);
{ keypress: RELEASE                                                            }
begin
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time

  if Key = aKeys[8].word then btn_jump_left_down.Visible := false else          // Jump (left)
  if Key = aKeys[9].word then btn_jump_right_down.Visible := false;             // Jump (right)
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if not bGameAPressed then
    begin
      bGameAPressed := true;

      if (iMode = 1) or (iMode = 7) or (iMode = 8) then                         // Time or Game Over -> HiScore B
        begin
          iMode := 3;
          Gam_Set_Mode;
        end;
    end;
end;





procedure TForm1.Inp_game_a_up;
{ game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;
  bGameAPressed := false;

  if iMode = 3 then                                                             // HiScore A -> Game A
    begin
      iMode := 5;
      Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_game_b_down;
{ game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if not bGameBPressed then
    begin
      bGameBPressed := true;

      if (iMode = 1) or (iMode = 7) or (iMode = 8) then                         // Time or Game Over -> HiScore B
        begin
          iMode := 4;
          Gam_Set_Mode;
        end;
    end;
end;





procedure TForm1.Inp_game_b_up;
{ game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;
  bGameBPressed := false;

  if iMode = 4 then                                                             // HiScore B -> Game B
    begin
      iMode := 6;
      Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_time_down;
{ time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if not bTimePressed then
    begin
      bTimePressed := true;

      if ((iMode >= 0) and (iMode < 3)) or (iMode = 7) or (iMode = 8) then      // any mode except hiscore/game modes ->
        begin
          Inc(iMode);

          if iMode > 2 then                                                     // -> Time -> Alarm ->
            iMode := 1;

          Gam_Set_Mode;
        end;
    end;
end;





procedure TForm1.Inp_time_up;
{ time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;
  bTimePressed := false;
end;





procedure TForm1.Inp_up_down;
{ up button: PRESS                                                             }
begin
  case iMode of
    0: begin                                                                    // ACL -> Time
         iMode := 1;
         Gam_Set_Mode;
       end;
    5, 6: if bControls then                                                     // Game A/B -> Move
            if ( (iHeroPosX = 3) and (iHeroPosY >= 1) and (iHeroPosY <= 4) )
            or ( (iHeroPosX = 9) and (iHeroPosY >= 1) and (iHeroPosY <= 5) ) then
              iDir := 1;
  end; // case..of
end;





procedure TForm1.Inp_left_down;
{ left button: PRESS                                                           }
begin
  case iMode of
    0: begin                                                                    // ACL -> Time
         iMode := 1;
         Gam_Set_Mode;
       end;
    5, 6: if bControls then                                                     // Game A/B -> Move
            if ( (iHeroPosX = 1) and (aIm_bonus_item[iHeroPosY].Visible) )
            or ( Odd(iHeroPosX) and (iHeroPosX > 1) ) then
              iDir := 2;
  end; // case..of
end;





procedure TForm1.Inp_right_down;
{ right button: PRESS                                                          }
begin
  case iMode of
    0: begin                                                                    // ACL -> Time
         iMode := 1;
         Gam_Set_Mode;
       end;
    5, 6: if bControls then                                                     // Game A/B -> Move
            if Odd(iHeroPosX) and (iHeroPosX < 11) then
              iDir := 3;
  end; // case..of
end;





procedure TForm1.Inp_down_down;
{ down button: PRESS                                                           }
begin
  case iMode of
    0: begin                                                                    // ACL -> Time
         iMode := 1;
         Gam_Set_Mode;
       end;
    5, 6: if bControls then                                                     // Game A/B -> Move
            if ( (iHeroPosX = 1)  and ( (iHeroPosY = 2) or (iHeroPosY = 4) ) )                    // (1, 2) or (1, 4) - ladders
            or ( (iHeroPosX = 4)  and ( (iHeroPosY = 2) or (iHeroPosY = 4) ) )                    // (4, 2) or (4, 4) - benches
            or ( (iHeroPosX = 8)  and ( (iHeroPosY = 1) or (iHeroPosY = 3) or (iHeroPosY = 5) ) ) // (8, 1) or (8, 3) or (8, 5) - benches
            or ( (iHeroPosX = 11) and ( (iHeroPosY = 3) or (iHeroPosY = 5) ) ) then               // (11, 1) or (11, 3) or (11, 5) - benches
              iDir := 4;
  end; // case..of
end;





procedure TForm1.Inp_jump_down;
{ any jump button: PRESS                                                       }
begin
  if (iMode = 5) or (iMode = 6) then
    if bControls and (iLoopPowerup = 0) then                                    // if hero is under control, and not in "powerup" mode
      if Odd(iHeroPosX) then                                                        // hero can jump only when on odd column
        if Odd(iHeroPosY) then                                                          // hero on row 1, 3 or 5
          iDir := 5                                                                         // jump left
        else                                                                            // hero on row 2 or 4
          iDir := 6;                                                                        // jump right
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  Timer_Alarm.Enabled          := false;
  Timer_Clock.Enabled          := false;
  Timer_Game.Enabled           := false;
  Timer_Game_Over.Enabled      := false;
  Timer_Game_Start.Enabled     := false;
  Timer_HiScore.Enabled        := false;
  Timer_Miss.Enabled           := false;
  Timer_Stage_Complete.Enabled := false;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/alarm/ACL mode, depending on the value of the iMode variable  }
  var
    i, j: integer;
begin
  case iMode of
    0: Vid_Show_All(true);                                                      // ACL
    1: begin                                                                    // TIME
         Gam_Stop_All_Timers;                                                           // stops all timers

         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_time_colon.Visible := true;                                           // displays colon

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;                                                   // clock starts ticking...
       end;
    2: begin                                                                    // ALARM
         Gam_Stop_All_Timers;                                                           // stops all timers

         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;                 // 12:00 (1)
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;                 // 12:00 (2)
         aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;                 // 12:00 (3)
         aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;                 // 12:00 (4)
         im_panel_time_colon.Visible := true;                                           // displays colon
         im_panel_bell.Visible := true;                                                 // displays bell

         Timer_Alarm.Interval := 500;
         Timer_Alarm.Enabled := true;                                                   // bell starts blinking
       end;
    3, 4: begin                                                                 // HISCORE A / B
         Gam_Stop_All_Timers;                                                           // stops all timers

         Vid_Show_All(false);                                                           // hides all items, chars, etc.

         case iMode of                                                                  // displays hiscore A / B
           3: Vid_Show_Panel_HiScore(iHiScoreA, true);
           4: Vid_Show_Panel_HiScore(iHiScoreB, true);
         end; // case..of

         iLoopHiScore := 1;                                                             // initializes "HiScore" timer
         Timer_HiScore.Interval := 1;
         Timer_HiScore.Enabled := true;                                                 // starts "GA" / "GB" blinking
         Timer_HiScore.Interval:= 500;                                                  // sets real interval to the "HiScore" timer
       end;
    5, 6: begin                                                                 // GAME A / B
         Gam_Stop_All_Timers;                                                           // stops all timers

         iScore := 0; Vid_Show_Panel_Score(iScore, true);                               // displays score (=0)
         i:= 0; Vid_Show_Panel_Lives(i);                                                // hides GA / GB if visible
         iScore := 0;                                                                   // initializes score
         iStage := 1;                                                                   // initializes current stage
         iLivesLeft := 2;                                                               // initializes lives left

         iLoopGameStart := 1;                                                           // initializes "Game Start" timer
         Timer_Game_Start.Interval:= 1;
         Timer_Game_Start.Enabled := true;                                              // starts "Game Start" timer
       end;
    7, 8: begin                                                                 // GAME OVER A / B
         iLoopGameOver := 1;                                                            // initializes "Game Over" timer
         Timer_Game_Over.Interval := 500;
         Timer_Game_Over.Enabled := true;                                               // starts "Game Over" timer
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Stage;
{ inits game variables for the very beginning of the game (score, lives left,  }
{ current stage, etc.)                                                         }
  var
    i, j: integer;
begin
  Vid_Show_Panel_Score(iScore, true);
  Vid_Show_Panel_Lives(iLivesLeft);
  iBonus := 500; i := ibonus div 100; Vid_Show_Panel_Bonus(i);

  Vid_Show_All_Kong(false); aIm_kong[5].Visible := true;
  aIm_woman[1].Visible := true;

  Vid_Show_All_Bonus_Item(true);

  for i := 0 to 12 do
    for j := 1 to 5 do
      aTire[i, j] := 0;

  iHeroPosX := 11; iHeroPosY := 1; Vid_Show_Hero(true);

  iLoopPowerup := 0;

  // sets difficulty
  case iMode of
    5:  case iStage of                                                          // Game A
          1: iMaxTires := 4;
          2: iMaxTires := 4;
          3: iMaxTires := 5;
          4: iMaxTires := 5;
          5: iMaxTires := 6;
          6: iMaxTires := 6;
          7: iMaxTires := 6;
          8: iMaxTires := 7;
          9: iMaxTires := 7;
         10: iMaxTires := 7;
         11: iMaxTires := 7;
         12: iMaxTires := 8;
       end; // case..of
    6:  case iStage of                                                          // Game B
          1: iMaxTires := 8;
          2: iMaxTires := 8;
          3: iMaxTires := 8;
          4: iMaxTires := 8;
          5: iMaxTires := 8;
          6: iMaxTires := 8;
          7: iMaxTires := 9;
          8: iMaxTires := 9;
          9: iMaxTires := 9;
         10: iMaxTires := 9;
         11: iMaxTires := 9;
         12: iMaxTires := 9;
       end; // case..of
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_AlarmTimer(Sender: TObject);
{ bell blinks at every 500 milliseconds                                        }
begin
  im_panel_bell.Visible := not im_panel_bell.Visible;                           // bell animation

  Timer_Alarm.Interval := 500;
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min div 10].Picture;       // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min mod 10].Picture;       // minute: units
  aIm_panel_digit_pos[5].Picture := nil;                                        // small digit

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i, j,
    iTires: integer;
    pfTemp: PFSoundSample;
    bMiss:  boolean;
begin
  // sets speed for next tick (slow if hero is moving or in "powerup mode", fast if hero is standing still)
  if (iDir = 0) then
    Timer_Game.Interval := 240
  else
    Timer_Game.Interval := 480;

  if iLoopPowerup <> 0 then
    Timer_Game.Interval := 480;

  // sets variables for later use
  bMiss := false;

  pfTemp := nil;

  // elapsed time (ticks) on current stage
  case iLoop of
     48: begin iBonus := 400; i := ibonus div 100; Vid_Show_Panel_Bonus(i); end;
    112: begin iBonus := 300; i := ibonus div 100; Vid_Show_Panel_Bonus(i); end;
    176: begin end;                                                             // bonus score starts blinking here
    208: bMiss := true;
  end; // case..of

  // aIm_panel_bonus sprite animation if iLoop > 176
  if iLoop > 176 then
    begin
      i := ibonus div 100;
      aIm_panel_bonus[i].Visible := not Odd(iLoop);                             // is displayed when "help" is hidden
    end;

  // woman "help" animation
  aIm_woman[2].Visible := Odd(iLoop);

  // checks collisions (part 1: with kong)
  bMiss := (iKongPos = 7) and (iHeroPosX = 5) and (iHeroPosY = 5);

  // checks collisions (part 2: with tires)
  if Odd(iHeroPosX) then
    begin
      if Odd(iHeroPosY)  and (aTire[iHeroPosX - 1, iHeroPosY] > 0) then         // on odd rows
        if iLoopPowerup = 0 then                                                    // is powerup mode active?
          bMiss := true                                                                 // no -> a miss has occured
        else
          begin                                                                         // yes -> breaks tire
            aTire[iHeroPosX - 1, iHeroPosY] := 0;
            Inc(iScore, 100); Vid_Show_Panel_Score(iScore, true);
          end;

      if (not Odd(iHeroPosY)) and (aTire[iHeroPosX + 1, iHeroPosY] > 0) then    // on even rows
        if iLoopPowerup = 0 then                                                    // is powerup mode active?
          bMiss := true                                                                 // no -> a miss has occured
        else
          begin                                                                         // yes -> breaks tire
            aTire[iHeroPosX + 1, iHeroPosY] := 0;
            Inc(iScore, 100); Vid_Show_Panel_Score(iScore, true);
          end;
    end;

  // tires movement + tires generation + kong animation (at every 2 ticks)
  if Odd(iLoop) then
    begin
      // movement
      for i := 12 downto 2 do if not Odd(i) then aTire[i, 1] := aTire[i - 2, 1];// row 1
      if (aTire[8, 2] = 2) then if Random(2) = 0 then                           // fast tire on (8, 2)
        begin aTire[8, 2] := 0; aTire[8, 1] := 1; end;
      if (aTire[2, 2] = 2) then if Random(2) = 0 then                           // fast tire on (2, 2)
        begin aTire[2, 2] := 0; aTire[2, 1] := 1; end;
      aTire[0, 1] :=  aTire[0, 2];                                              // tire on (0, 2)

      for i := 0 to 10 do if not Odd(i) then aTire[i, 2] := aTire[i + 2, 2];    // row 2
      if (aTire[4, 3] = 2) then if Random(2) = 0 then                           // fast tire on (4, 3)
        begin aTire[4, 3] := 0; aTire[4, 2] := 2; end;
      if (aTire[10, 3] = 2) then if Random(2) = 0 then                          // fast tire on (10, 3)
        begin aTire[10, 3] := 0; aTire[10, 2] := 2; end;
      aTire[12, 2] :=  aTire[12, 3];                                            // any tire on (12, 3)

      for i := 12 downto 2 do if not Odd(i) then aTire[i, 3] := aTire[i - 2, 3];// row 3
      if (aTire[8, 4] = 2) then if Random(2) = 0 then                           // fast tire on (8, 4)
        begin aTire[8, 4] := 0; aTire[8, 3] := 2; end;
      if (aTire[2, 4] = 2) then if Random(2) = 0 then                           // fast tire on (2, 4)
        begin aTire[2, 4] := 0; aTire[2, 3] := 2; end;
      aTire[0, 3] :=  aTire[0, 4];                                              // tire on (0, 4)

      for i := 0 to 10 do if not Odd(i) then aTire[i, 4] := aTire[i + 2, 4];    // row 4
      if (aTire[4, 5] = 2) then if Random(2) = 0 then                           // fast tire on (4, 5)
        begin aTire[4, 5] := 0; aTire[4, 4] := 2; end;
      if (aTire[10, 5] = 2) then if Random(2) = 0 then                          // fast tire on (10, 5)
        begin aTire[10, 5] := 0; aTire[10, 4] := 2; end;
      aTire[12, 4] :=  aTire[12, 5];                                            // any tire on (12, 5)

      for i := 12 downto 4 do if not Odd(i) then aTire[i, 5] := aTire[i - 2, 5];// row 5

      // generation
      iTires := 0;                                                              // first of all, count tires on screen

      if (atire[2, 5] > 0) then Inc(iTires);
      for i := 4 to 12 do if (not Odd(i)) and (atire[i, 5] > 0) then Inc(iTires);
      for i := 0 to 12 do for j := 1 to 4 do if (not Odd(i)) and (atire[i, j] > 0) then Inc(iTires);

      if iKongPos = 6 then                                                      // if kong is in 6 (and generated new tire on previous tick)
        begin
          if Random(3) = 0 then                                                     // places kong in 5 (67% chance) or 7 (33% chance)
            iKongPos := 7
          else
            iKongPos := 5;

          aTire[2, 5] := 0;                                                         // clears tire in (2, 5)
        end
      else                                                                      // else
        if iTires < iMaxTires then
          begin
            if (Random(3) = 0) or (iTires = 0) then                                 // 33% chance (or 100% if there's no tire on screen)
              begin
                iKongPos := 6;                                                      // places kong in 6
                aTire[2, 5] := 1;                                                   // generates slow tire
              end;
          end;

      // forces tires and kong on ticks 1 and 3
      case iLoop of
        1: begin                                                                // first tick
             iKongPos := 6;                                                         // places kong in 6
             aTire[2, 5] := 2;                                                      // generates fast tire

             if iMode = 6 then                                                      // on Game B
               aTire[10, 5] := 2;                                                       // generates additional fast tire in (10, 5)
           end;
        3: begin
             iKongPos := 7;                                                             // places kong in 7
             aTire[2, 5] := 0;                                                          // clears tire in (2, 5)
           end;
      end; // case..of

      // displays tires
      aIm_tire[2, 5].Visible := atire[2, 5] > 0;

      for i := 4 to 12 do
        if (not Odd(i)) then aIm_tire[i, 5].Visible := atire[i, 5] > 0;

      for i := 0 to 12 do
        for j := 1 to 4 do
          if (not Odd(i)) then aIm_tire[i, j].Visible := atire[i, j] > 0;

      // displays kong
      for i := 5 to 7 do
        aIm_kong[i].Visible := (i = iKongPos);
    end;

  // hero moves
  if not bMiss then
    case iDir of
      1: if iLoopPowerup = 0 then                                               // UP (if not "powerup" mode)
           begin
             if ((iHeroPosX = 3) and (iHeroPosY = 1)) or                            // hero climbs ladder (to left, part 1)
                ((iHeroPosX = 3) and (iHeroPosY = 3)) then
               begin Vid_Show_Hero(false); Dec(iHeroPosX); Vid_Show_Hero(true); end
             else
             if ((iHeroPosX = 2) and (iHeroPosY = 1)) or                            // hero climbs ladder (to left, part 2)
                ((iHeroPosX = 2) and (iHeroPosY = 3)) then
               begin Vid_Show_Hero(false); Dec(iHeroPosX); Inc(iHeroPosY); Vid_Show_Hero(true); iDir := 0; pfTemp := pfs_moveup; end
             else
             if ((iHeroPosX = 9) and (iHeroPosY = 2)) or                            // climbs ladder (to right, part 1)
                ((iHeroPosX = 9) and (iHeroPosY = 4)) then
               begin Vid_Show_Hero(false); Inc(iHeroPosX); Vid_Show_Hero(true); end
             else
             if ((iHeroPosX = 10) and (iHeroPosY = 2)) or                           // hero climbs ladder (to right, part 2)
                ((iHeroPosX = 10) and (iHeroPosY = 4)) then
               begin Vid_Show_Hero(false); Inc(iHeroPosX); Inc(iHeroPosY); Vid_Show_Hero(true); iDir := 0; pfTemp := pfs_moveup; end
             else
             if ((iHeroPosX = 9) and (iHeroPosY = 1)) or                            // hero climbs bench (to left)
                ((iHeroPosX = 9) and (iHeroPosY = 3)) or
                ((iHeroPosX = 9) and (iHeroPosY = 5)) then
               begin Vid_Show_Hero(false); Dec(iHeroPosX); Vid_Show_Hero(true); iDir := 0; pfTemp := pfs_moveup; end
             else
             if ((iHeroPosX = 3) and (iHeroPosY = 2)) or                            // hero climbs bench (to right)
                ((iHeroPosX = 3) and (iHeroPosY = 4)) then
               begin Vid_Show_Hero(false); Inc(iHeroPosX); Vid_Show_Hero(true); iDir := 0; pfTemp := pfs_moveup; end;
           end;
      2: if Odd(iHeroPosX) then                                                 // LEFT
           if (iHeroPosX = 5) and (iHeroPosY = 5) then                              // is hero in front of kong?
             begin                                                                      // stage complete!
               Gam_Stop_All_Timers;
               bControls := false;

               Vid_Show_Hero(false); Dec(iHeroPosX, 4); Vid_Show_Hero(true);
               iDir := 0;
               pfTemp := pfs_move;

               iLoopStageComplete := 1;
               Timer_Stage_Complete.Interval := 100;
               Timer_Stage_Complete.Enabled := true;
             end
           else                                                                     // else...
             begin
               if iHeroPosX > 1 then

                 if aTire[iHeroPosX - 1, iHeroPosY] = 0 then                            // if there's no tire on the left
                   begin                                                                    // hero moves
                     Vid_Show_Hero(false); Dec(iHeroPosX, 2); Vid_Show_Hero(true);
                     iDir := 0;
                     pfTemp := pfs_move;
                   end
                 else                                                                   // else
                   if iLoopPowerup = 0 then                                                 // is powerup mode active?
                     bMiss := true                                                              // no -> a miss has occured
                   else
                     begin                                                                      // yes -> breaks tire
                       aTire[iHeroPosX - 1, iHeroPosY] := 0;
                       Inc(iScore, 100); Vid_Show_Panel_Score(iScore, true);
                     end
               else
                 if iHeroPosX = 1 then                                                  // chance to get bonus item
                   begin
                     if aIm_bonus_item[iHeroPosY].Visible then
                       if Odd(iHeroPosY) then                                               // wallet or shoe
                         begin
                           aIm_bonus_item[iHeroPosY].Visible := false;
                           Inc(iScore, 100);
                           Vid_Show_Panel_Score(iScore, true);
                           iDir := 0;
                           pfTemp := pfs_move;
                         end
                       else                                                                 // cake (powerup)
                         begin
                           aIm_bonus_item[iHeroPosY].Visible := false;
                           iLoopPowerup := 1;
                           iDir := 0;
                         end;
                   end;
             end;
      3: if Odd(iHeroPosX) then                                                 // RIGHT
           if (iHeroPosX < 11) then
             if aTire[iHeroPosX + 1, iHeroPosY] = 0 then                            // if there's no tire on the right
               begin                                                                    // hero moves
                 Vid_Show_Hero(false); Inc(iHeroPosX, 2); Vid_Show_Hero(true);
                 iDir := 0;
                 pfTemp := pfs_move;
               end
             else                                                                    // else
               if iLoopPowerup = 0 then                                                  // is powerup mode active?
                 bMiss := true                                                               // no -> a miss has occured
               else
                 begin                                                                       // yes -> breaks tire
                   aTire[iHeroPosX + 1, iHeroPosY] := 0;
                   Inc(iScore, 100); Vid_Show_Panel_Score(iScore, true);
                 end;
      4: if iLoopPowerup = 0 then                                               // DOWN (if not "powerup" mode)
           begin
             if ((iHeroPosX = 1) and (iHeroPosY = 2)) or                            // hero descends ladder (to right, part 1)
                ((iHeroPosX = 1) and (iHeroPosY = 4)) then
               begin Vid_Show_Hero(false); Inc(iHeroPosX); Dec(iHeroPosY); Vid_Show_Hero(true); end
             else
             if ((iHeroPosX = 2) and (iHeroPosY = 1)) or                            // hero descends ladder (to right, part 2)
                ((iHeroPosX = 2) and (iHeroPosY = 3)) then
               begin Vid_Show_Hero(false); Inc(iHeroPosX); Vid_Show_Hero(true); iDir := 0; pfTemp := pfs_moveup; end
             else
             if ((iHeroPosX = 11) and (iHeroPosY = 3)) or                           // hero descends ladder (to left, part 1)
                ((iHeroPosX = 11) and (iHeroPosY = 5)) then
               begin Vid_Show_Hero(false); Dec(iHeroPosX); Dec(iHeroPosY); Vid_Show_Hero(true); end
             else
             if ((iHeroPosX = 10) and (iHeroPosY = 2)) or                           // hero descends ladder (to left, part 2)
                ((iHeroPosX = 10) and (iHeroPosY = 4)) then
               begin Vid_Show_Hero(false); Dec(iHeroPosX); Vid_Show_Hero(true); iDir := 0; pfTemp := pfs_moveup; end
             else
             if ((iHeroPosX = 8) and (iHeroPosY = 1)) or                            // hero descends bench (to left)
                ((iHeroPosX = 8) and (iHeroPosY = 3)) or
                ((iHeroPosX = 8) and (iHeroPosY = 5)) then
               begin Vid_Show_Hero(false); Dec(iHeroPosX); Vid_Show_Hero(true); iDir := 0; pfTemp := pfs_moveup; end
             else
             if ((iHeroPosX = 4) and (iHeroPosY = 2)) or                            // hero descends bench (to right)
                ((iHeroPosX = 4) and (iHeroPosY = 4)) then
               begin Vid_Show_Hero(false); Inc(iHeroPosX); Vid_Show_Hero(true); iDir := 0; pfTemp := pfs_moveup; end;
           end;
      5: if iHeroPosX > 1 then                                                  // JUMP LEFT
           begin
             bControls := false;                                                    // prevents player from operating hero while he's jumping

             if not Odd(iHeroPosX) then                                             // hero stops jumping (to left, part 2 only)
               begin
                 iDir := 0;
                 pfTemp := pfs_jump;
                 bControls := true;                                                 // hero can be operated again at the end of his jump
               end;

             if Odd(iHeroPosX) then                                                 // jumping a tire?
               begin
                 // jumping a tire? (part 1, long jump)
                 if (aTire[iHeroPosX - 3, iHeroPosY] > 0) and (aTire[iHeroPosX - 3, iHeroPosY] < 3) then
                   begin
                     aTire[iHeroPosX - 3, iHeroPosY] := 3;
                     Inc(iScore, 10);
                     Vid_Show_Panel_Score(iScore, true);
                   end;

                 // jumping a tire? (part 2, short jump)
                 if (aTire[iHeroPosX - 1, iHeroPosY] > 0) and (aTire[iHeroPosX - 1, iHeroPosY] < 3) then
                   begin
                     aTire[iHeroPosX - 1, iHeroPosY] := 3;
                     Inc(iScore, 10);
                     Vid_Show_Panel_Score(iScore, true);
                   end;
               end;

             Vid_Show_Hero(false); Dec(iHeroPosX); Vid_Show_Hero(true);             // hero jumps (to left, part 1 and 2)
           end;
      6: if iHeroPosX < 11 then                                                 // JUMP RIGHT
           begin
             bControls := false;                                                    // prevents player from operating hero while he's jumping

             if not Odd(iHeroPosX) then                                             // hero stops jumping (to left, part 2 only)
               begin
                 iDir := 0;
                 pfTemp := pfs_jump;
                 bControls := true;                                                 // hero can be operated again at the end of his jump
               end;

             if Odd(iHeroPosX) then                                                 // jumping a tire?
               begin
                 // jumping a tire? (part 1, long jump)
                 if (aTire[iHeroPosX + 3, iHeroPosY] > 0) and (aTire[iHeroPosX + 3, iHeroPosY] < 3) then
                   begin
                     aTire[iHeroPosX + 3, iHeroPosY] := 3;
                     Inc(iScore, 10);
                     Vid_Show_Panel_Score(iScore, true);
                   end;

                 // jumping a tire? (part 2, short jump)
                 if (aTire[iHeroPosX + 1, iHeroPosY] > 0) and (aTire[iHeroPosX + 1, iHeroPosY] < 3) then
                   begin
                     aTire[iHeroPosX + 1, iHeroPosY] := 3;
                     Inc(iScore, 10);
                     Vid_Show_Panel_Score(iScore, true);
                   end;
               end;

             Vid_Show_Hero(false); Inc(iHeroPosX); Vid_Show_Hero(true);             // hero jumps (to right, part 1 and 2)
           end;
    end; // case..of

  // checks for misses, else game continues
  if bMiss then
    begin
      // a miss has occured!
      Gam_Stop_All_Timers;
      bControls := false;

      iLoopMiss := 1;
      Timer_Miss.Interval := 1;
      Timer_Miss.Enabled := true;
    end
  else
    begin
      // sound management
      if iLoopPowerup > 0 then                                                  // if "powerup" mode
        Snd_Play(pfs_powerup)                                                       // plays pfPowerup
      else                                                                      // else
        if pfTemp <> nil then                                                       // plays pfMove, pfJump or nothing
          Snd_Play(pfTemp);

      // loops action (powerup)
      if iLoopPowerup > 0 then
        Inc(iLoopPowerup);

      if iLoopPowerup > 15 then                                                 // "powerup" mode ends
        iLoopPowerup := 0;

      // loops action (overall)
      Inc(iLoop);
    end;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ GA or GB blinks as long as no key is pressed for around 25 seconds           }
  var
    i: integer;
begin
  if iLoopGameOver < 50 then                                                    // blinking
    if Odd(iLoopGameOver) then                                                      // displays GA / GB

      if iMode = 7 then
        aIm_panel_lives[1].Visible := true
      else
        aIm_panel_lives[3].Visible := true

    else                                                                            // hides GA / GB
      begin
         i := 0;
         Vid_Show_Panel_Lives(i);
      end
  else                                                                          // back to "time" mode
    begin
      iMode := 1;
      Gam_Set_Mode;

      Timer_Game_Over.Enabled := false;
    end;

  Inc(iLoopGameOver);                                                           // loops timer
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ kong climbs up the tree with woman; start tune plays; then stage starts      }
  var
    i: integer;
begin
  case iLoopGameStart of
   1: begin
        i := 0; Vid_Show_Panel_Score(i, false);
        aIm_kong[2].Visible := true;

        Snd_Play(pfs_start);

        Timer_Game_Start.Interval := 2800;
      end;
   2: begin
        Vid_Show_All_Kong(false); aIm_kong[3].Visible := true;

        Timer_Game_Start.Interval := 200;
      end;
   3: begin
        Vid_Show_All_Kong(false); aIm_kong[4].Visible := true;

        Timer_Game_Start.Interval := 370;
      end;
   4: begin
        Vid_Show_All_Kong(false); aIm_kong[5].Visible := true;
        aIm_woman[1].Visible := true;

        Timer_Game_Start.Interval := 750;
      end;
   5: begin
        Gam_Set_Stage;
        iDir := 0;
        bControls := true;

        iLoop := 1;
        Timer_Game.Interval := 480;
        Timer_Game.Enabled := true;

        Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGameStart);                                                          // loops timer
end;





procedure TForm1.Timer_HiScoreTimer(Sender: TObject);
{ GA or GB blinks as long as Game A or Game B buttons are pressed              }
  var
    i: integer;
begin
  if Odd(iLoopHiScore) then                                                     // displays GA / GB

    if iMode = 3 then
      aIm_panel_lives[1].Visible := true
    else
      aIm_panel_lives[3].Visible := true

  else                                                                          // hides GA / GB
    begin
       i := 0;
       Vid_Show_Panel_Lives(i);
    end;

  Inc(iLoopHiScore);                                                            // loops timer

  if iLoopHiScore > 2 then
    iLoopHiScore := 1;
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ decreases lives left, then game reprises or is over                          }
begin
  case iLoopMiss of
    1: begin
         Snd_Play(pfs_miss);

         Timer_Miss.Interval := 300;
       end;
    2: begin
         Vid_Show_All(false);
         im_hero_miss.Visible := true;

         Timer_Miss.Interval := 970;
       end;
    3: begin
         Dec(iLivesLeft);

         if iLivesLeft >= 0 then
           Timer_Miss.Interval := 500                                            // game reprises (part 1)
         else
           begin                                                                // game is over (part 1)
             if (iMode = 5) and (iScore > iHiScoreA) then begin iHiScoreA := iScore; iReg.WriteInteger('', 'HiScore1', iHiScoreA); end;
             if (iMode = 6) and (iScore > iHiScoreB) then begin iHiScoreB := iScore; iReg.WriteInteger('', 'HiScore2', iHiScoreB); end;
             Snd_Play(pfs_gameover);

             Timer_Miss.Interval := 3520;
           end;
       end;
    4: begin
         if iLivesLeft >= 0 then
           begin                                                                // game reprises (part 2)
             Gam_Set_Stage;
             im_hero_miss.Visible := false;
             iDir := 0;
             bControls := true;

             iLoop := 1;
             Timer_Game.Interval := 480;
             Timer_Game.Enabled := true;
           end
         else
           begin                                                                // game is over (part 2)
             Inc(iMode, 2);
             Gam_Set_Mode;
           end;

         Timer_Miss.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopMiss);
end;





procedure TForm1.Timer_Stage_CompleteTimer(Sender: TObject);
{ stage is complete. kong falls from tree at every 4 stages; bonus life given  }
{ at stage 2 and at every 4 stages                                             }
  var
    i: integer;
begin
  case iLoopStageComplete of
    1: begin
         Snd_Play(pfs_bonus1);

         Timer_Stage_Complete.Interval := 1400;
       end;
    2: begin
         Vid_Show_All_Bonus_Item(false);
         Vid_Show_All_Panel_Bonus(false);
         Vid_Show_All_Tire(false);
         Vid_Show_All_Woman(false);
         i := 0; Vid_Show_Panel_Lives(i);
         aIm_kong[6].Visible  := false; aIm_kong[5].Visible  := true;
         aIm_woman[3].Visible := true;  aIm_woman[4].Visible := true;

         Inc(iScore, iBonus); Vid_Show_Panel_Score(iScore, true);

         Timer_Stage_Complete.Interval := 1600;
      end;
    3: begin
         Inc(iStage);

         if iStage >= 13 then                                                   // loops stage to 9-12 if higher than 13
           iStage := 9;

         if (iStage - 1) mod 4 = 0 then                                         // at stage 4 and at every 4 stages
           iLoopStageComplete := 4                                                  // kong falls down from tree (goes case 5)
         else
           begin
             aIm_woman[4].Visible := false; aIm_woman[5].Visible := true;
             aIm_kong[5].Visible  := false; aIm_kong[6].Visible  := true;
           end;

         if (iStage + 1) mod 4 = 0 then                                         // at stage 2 and at every 4 stages
           Inc(iLivesLeft);                                                         // bonus life given

         Timer_Stage_Complete.Interval := 550;
       end;
    4: begin
         aIm_woman[3].Visible := false; aIm_woman[5].Visible := false;
         aIm_kong[6].Visible  := false;
         aIm_hero[1, 5].Visible := false;

         iLoopGameStart := 1;                                                   // initializes "Game Start" timer
         Timer_Game_Start.Interval:= 1;
         Timer_Game_Start.Enabled := true;                                      // starts "Game Start" timer

         Timer_Stage_Complete.Enabled := false;                                 // (end of first animation)
       end;
    // kong falls down from tree
    5: begin aIm_kong[5].Visible := false; aIm_kong[4].Visible := true; Timer_Stage_Complete.Interval := 210; Snd_Play(pfs_bonus2); end;
    6: begin aIm_kong[4].Visible := false; aIm_kong[3].Visible := true; end;
    7: begin aIm_kong[3].Visible := false; aIm_kong[2].Visible := true; end;
    8: begin aIm_kong[2].Visible := false; aIm_kong[1].Visible := true; Timer_Stage_Complete.Interval := 2100; end;
    9: begin
         aIm_woman[3].Visible := false; aIm_woman[4].Visible := false;
         aIm_kong[1].Visible  := false;
         aIm_hero[1, 5].Visible := false;

         iLoopGameStart := 1;                                                   // initializes "Game Start" timer
         Timer_Game_Start.Interval:= 1;
         Timer_Game_Start.Enabled := true;                                      // starts "Game Start" timer

         Timer_Stage_Complete.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopStageComplete);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_controller.Checked := bDisplayCtrlOverlays;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_controllerClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_controller.Checked := not pm_12_controller.Checked;
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

{}iReg.WriteBool('', 'CtrlrOverlay', pm_12_controller.Checked);

  Vid_Display_Ctrl_Overlay;
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
